from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, load_learning_effect
import pandas as pd
import numpy as np
from scipy.stats import kruskal, mannwhitneyu

def calc_F1(predictions, ground_truth, name="benign"):
	predictions_ = map_item_list(predictions)
	ground_truth_ = map_item_list(ground_truth)
	corect_number = 0
	for item in predictions_:
		if item in ground_truth_:
			corect_number += 1
	if corect_number == 0:
		precision = 0.0
		recall = 0.0
		F1 = 0.0
		# print("-" * 17)
		# print(name, corect_number)
		# print("prediction", predictions_)
		# print("ground_truth", ground_truth_)
	else:
		precision = corect_number / float(len(predictions_))
		recall = corect_number / float(len(ground_truth_))
		F1 = (2 * precision * recall) / (precision + recall)
	return precision, recall, F1

def check_features(predictions):
	malignant_ground_truth = ['Blue-Whitish Veil', 'Regression Structures', 'Streaks - type 1', 'Dots & Globules - type 2', 'Pigment Network - type 1']
	benign_ground_truth = ['Streaks - type 2', 'Dots & Globules - type 1', 'Pigment Network - type 2']
	for item in predictions:
		if item not in malignant_ground_truth and item not in benign_ground_truth:
			raise NotImplementedError(f"Unknown concept {item}")

name_map = {'Blue-Whitish Veil': 1, 
			'Dots & Globules - type 1': 2, 
			'Dots & Globules - type 2': 3, 
			'Regression Structures': 4,
 			'Streaks - type 1': 5,
 			'Streaks - type 2': 6,
 			'Pigment Network - type 1': 7,
 			'Pigment Network - type 2': 8
 }

def map_item_list(tp_list):
	# print(type(tp_list[0]))
	new_list = [name_map[item] for item in tp_list]
	return new_list

def calc_user_learning_effect(valid_users, user_malignant_features, user_benign_features, userinfo_dict):
	malignant_ground_truth = ['Blue-Whitish Veil', 'Regression Structures', 'Streaks - type 1', 'Dots & Globules - type 2', 'Pigment Network - type 1']
	benign_ground_truth = ['Streaks - type 2', 'Dots & Globules - type 1', 'Pigment Network - type 2']
	performance_groups = {}
	# performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	for user in valid_users:
		if userinfo_dict[user] == "control":
			continue
		malignant_features = user_malignant_features[user]
		benign_features = user_benign_features[user]
		p_, r_, F_malignant = calc_F1(predictions=malignant_features, ground_truth=malignant_ground_truth, name="malignant")
		p_, r_, F_benign = calc_F1(predictions=benign_features, ground_truth=benign_ground_truth, name="benign")
		user_group = userinfo_dict[user]
		# F1_avg = (F_malignant + F_benign) / 2
		F1_avg = F_malignant * (5 / 8) + F_benign * (3 / 8)
		performance_groups[user_group].append({"F1_malignant": F_malignant, "F1_benign": F_benign, "F1_avg": F1_avg})
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))

	def get_performance_dict(performance_list):
		keys = ['F1_malignant', 'F1_benign', 'F1_avg']
		performance_dict = {}
		for key_ in keys:
			performance_dict[key_] = []
		for tp_performance in performance_list:
			for key_ in keys:
				performance_dict[key_].append(tp_performance[key_])
		return performance_dict

	def compare_all_groups(performance_groups, compared_groups=["control", "concept"]):
		performance_dict = {}
		for group in compared_groups:
			performance_dict[group] = get_performance_dict(performance_groups[group])
		keys = ['F1_malignant', 'F1_benign', 'F1_avg']
		for var_name in keys:
			for group in compared_groups:
				print(len(performance_dict[group][var_name]))
			kwargs = [performance_dict[group][var_name] for group in compared_groups]
			print(var_name)
			print(kwargs[0])
			print(kwargs[1])
			statistic, pvalue = kruskal(*kwargs)
			print(var_name)
			print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
			for group in compared_groups:
				data_list_1 = performance_dict[group][var_name]
				print("Mean: M({}):{:.2f}, SD({}):{:.2f}".format(group, np.mean(data_list_1), group, np.std(data_list_1)))
			if pvalue < 0.05 / 4:
				length = len(compared_groups)
				for i in range(length - 1):
					for j in range(i+1, length):
						group_1 = compared_groups[i]
						group_2 = compared_groups[j]
						data_list_1 = performance_dict[group_1][var_name]
						data_list_2 = performance_dict[group_2][var_name]
						post_hoc_comparison(data_list_1, data_list_2, group_1, group_2)
			print("-" * 17)
	compare_all_groups(performance_groups, compared_groups=["concept", "archie_target", "archie", "archie_od"])
	return performance_groups

# H1, understanding; through postQ_ham concepts F1? Recall?

if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	user_malignant_features, user_benign_features = load_learning_effect(userinfo_dict, valid_users)

	calc_user_learning_effect(valid_users, user_malignant_features, user_benign_features, userinfo_dict)
