from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance
from scipy.stats import kruskal, mannwhitneyu

def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict

def post_hoc_comparison(data_list_1, data_list_2, name1, name2):
	print("Use pots-hoc analysis")
	threshold = 0.05 / 4
	flag = False
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='greater')
	if pvalue < threshold:
		print("Alternative {} > {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='less')
	if pvalue < threshold:
		print("Alternative {} < {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	if not flag:
		print("No significant difference with post-hoc analysis")

def get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list):
	num_tasks = 14.0
	performance_groups = {}
	performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	for user in valid_users:
		tp_performance = UserPerformance(username=user)
		user_group = userinfo_dict[user]

		tp_correct, tp_agreement_fraction, tp_switching_fraction, tp_appropriate_reliance, initial_disagreement, relative_positive_ai_reliance, relative_positive_self_reliance = \
		calc_user_reliance_measures(user, usertask_dict=new_user_task_dict, answer_dict=answer_dict, task_id_list=task_ID_list)
		tp_accuracy = tp_correct / num_tasks

		tp_performance.add_performance(accuracy=tp_accuracy, agreement_fraction=tp_agreement_fraction, switching_fraction=tp_switching_fraction, 
			appropriate_reliance=tp_appropriate_reliance, relative_positive_ai_reliance=relative_positive_ai_reliance, 
			relative_positive_self_reliance=relative_positive_self_reliance, group="overall")
		performance_groups[user_group].append(tp_performance)
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))

	def get_performance_dict(performance_list):
		keys = ["accuracy", "agreement_fraction", "switching_fraction", "appropriate_reliance", "relative_positive_ai_reliance", "relative_positive_self_reliance"]
		performance_dict = {}
		for key_ in keys:
			performance_dict[key_] = []
		# performance_dict["trust"] = []
		for tp_performance in performance_list:
			for key_ in keys:
				performance_dict[key_].append(tp_performance.performance[key_])
				# we only look at the first batch for H1
			# performance_dict["trust"].append(user_trust_first[tp_performance.username])
		return performance_dict

	def compare_all_groups(performance_groups, compared_groups=["control", "concept"]):
		performance_dict = {}
		for group in compared_groups:
			performance_dict[group] = get_performance_dict(performance_groups[group])
		keys = ["accuracy", "agreement_fraction", "switching_fraction", "appropriate_reliance", "relative_positive_ai_reliance", "relative_positive_self_reliance"]
		for var_name in keys:
			# data_list_1 = performance_dict_1[var_name]
			# data_list_2 = performance_dict_2[var_name]
			# data_list_3 = performance_dict_3[var_name]
			# print(len(data_list_1), len(data_list_3), len(data_list_2))
			for group in compared_groups:
				print(len(performance_dict[group][var_name]))
			kwargs = [performance_dict[group][var_name] for group in compared_groups]
			statistic, pvalue = kruskal(*kwargs)
			print(var_name)
			print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
			for group in compared_groups:
				data_list_1 = performance_dict[group][var_name]
				print("Mean: M({}):{:.2f}, SD({}):{:.2f}".format(group, np.mean(data_list_1), group, np.std(data_list_1)))
			if pvalue < 0.05 / 4:
				length = len(compared_groups)
				for i in range(length - 1):
					for j in range(i+1, length):
						group_1 = compared_groups[i]
						group_2 = compared_groups[j]
						data_list_1 = performance_dict[group_1][var_name]
						data_list_2 = performance_dict[group_2][var_name]
						post_hoc_comparison(data_list_1, data_list_2, group_1, group_2)
			print("-" * 17)
	compare_all_groups(performance_groups, compared_groups=["control", "concept", "archie_target", "archie", "archie_od"])
	# return performance_groups

if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	condition_count = {}
	for user in valid_users:
		# if user not in approved_participants:
		# 	continue
		user_group = userinfo_dict[user]
		if user_group not in condition_count:
			condition_count[user_group] = 0
		condition_count[user_group] += 1
	for condition in condition_count:
		print(condition, condition_count[condition])

	# standarize the user choice dict and answer dict
	new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
	get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list)











