from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, load_expertise, load_nasatlx
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance, calc_TiA_scale, calc_ATI_scale
from scipy.stats import kruskal, mannwhitneyu
from pingouin import ancova, anova
from statsmodels.stats.multicomp import pairwise_tukeyhsd

def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict

def post_hoc_comparison(data_list_1, data_list_2, name1, name2):
	print("Use pots-hoc analysis")
	threshold = 0.05 / 4
	flag = False
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='greater')
	if pvalue < threshold:
		print("Alternative {} > {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='less')
	if pvalue < threshold:
		print("Alternative {} < {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	if not flag:
		print("No significant difference with post-hoc analysis")

def get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list):
	num_tasks = 14.0
	performance_groups = {}
	performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	user2performance = {}
	for user in valid_users:
		tp_performance = UserPerformance(username=user)
		user_group = userinfo_dict[user]

		tp_correct, tp_agreement_fraction, tp_switching_fraction, tp_appropriate_reliance, initial_disagreement, relative_positive_ai_reliance, relative_positive_self_reliance = \
		calc_user_reliance_measures(user, usertask_dict=new_user_task_dict, answer_dict=answer_dict, task_id_list=task_ID_list)
		tp_accuracy = tp_correct / num_tasks

		tp_performance.add_performance(accuracy=tp_accuracy, agreement_fraction=tp_agreement_fraction, switching_fraction=tp_switching_fraction, 
			appropriate_reliance=tp_appropriate_reliance, relative_positive_ai_reliance=relative_positive_ai_reliance, 
			relative_positive_self_reliance=relative_positive_self_reliance, group="overall")
		user2performance[user] = tp_performance
		performance_groups[user_group].append(tp_performance)
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))
	return user2performance





if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
	

	user2performance = get_all_reliance_measures(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list)
	user_ATI_scale, user_ATI_time = calc_ATI_scale()
	user_TiA_scale = calc_TiA_scale()
	user_expertise = load_expertise(valid_users)
	user_cognitive_load = load_nasatlx(valid_users)
	# TiA_subscales = ["Reliability/Competence", "Understanding/Predictability", "Familiarity", \
	# 			 "Intention of Developers", "Propensity to Trust", "Trust in Automation"]
	variable_dict = {
		# "user_id": [],
		"ATI": [],
		# "TiA-Propensity": [],
		# "Tutorial": [],
		# "XAI": [],
		"medical_expertise": [],
		"skin_cancer_expertise": [],
		"cognitive_load": [],
		"condition": [],
		# "Accuracy": [],
		# "Agreement_fraction": [],
		# "switching_fraction": [],
		# "Accuracy-wid": [],
		# "RAIR": [],
		# "RSR": []
	}
	nasas_dimensions = ['mental_demand', 'physical_demand', 'temporal_demand', 'performance', 'effort', 'frustration']
	for subscale in nasas_dimensions:
		variable_dict[subscale] = []

	for user in valid_users:
		# variable_dict["user_id"].append(user)
		# variable_dict["TiA-Propensity"].append(user_ptt_scale[user])
		tp_condition = userinfo_dict[user]
		variable_dict["condition"].append(tp_condition)
		# print(user, tp_condition)

		variable_dict["ATI"].append(user_ATI_scale[user])
		variable_dict["medical_expertise"].append(user_expertise[user]["medical_expertise"])
		variable_dict["skin_cancer_expertise"].append(user_expertise[user]["skin_cancer_expertise"])
		
		variable_dict["cognitive_load"].append(user_cognitive_load[user]['avg_cognitive_load'])

		# TiA subscales
		for subscale in nasas_dimensions:
			variable_dict[subscale].append(user_cognitive_load[user][subscale])

	df = pd.DataFrame(variable_dict)
	print(df.shape)
	# print(df['condition'])
	# print("unique", df['condition'].unique())
	# print(df)
	# print(df.isnull().values.any())

	print("-" * 34)

	print("For all participants, compare with experimental conditions")
	aov = anova(dv='cognitive_load', between='condition', data=df, effsize='n2')
	print(aov.round(3))


	# we found significant impact of condition on cognitive_load, use post-hoc analysis:
	tukey = pairwise_tukeyhsd(endog=df['cognitive_load'], groups=df['condition'], alpha=0.0125)
	print(tukey)

	for var_name in nasas_dimensions:
		print(var_name)
		avo_ = anova(dv=var_name, between='condition', data=df, effsize='n2')
		# print(ancova(data=df, dv='cognitive_load', covar='ATI', between='condition', effsize='n2'))
		print(avo_.round(3))
		print("-" * 34)
		tukey = pairwise_tukeyhsd(endog=df[var_name], groups=df['condition'], alpha=0.0125)
		print(tukey)
	# for var_name in ["Accuracy", "Agreement_fraction", "switching_fraction", "Accuracy-wid", "RAIR", "RSR"]:
	# 	print(var_name)
	# 	avo_ = ancova(data=df, dv=var_name, covar=['medical_expertise', 'skin_cancer_expertise', 'ATI', 'Propensity to Trust'], between='condition', effsize='n2')
	# 	# print(ancova(data=df, dv='cognitive_load', covar='ATI', between='condition', effsize='n2'))
	# 	print(avo_.round(3))
	# 	print("-" * 34)













