from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, get_user_final_task_time, compute_task_time
import pandas as pd
import numpy as np
from scipy.stats import kruskal, mannwhitneyu
from calc_measures import calc_ATI_scale


def post_hoc_comparison(data_list_1, data_list_2, name1, name2):
	print("Use pots-hoc analysis")
	threshold = 0.05 / 4
	flag = False
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='greater')
	if pvalue < threshold:
		print(statistic, pvalue)
		print("Alternative {} > {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='less')
	if pvalue < threshold:
		print(statistic, pvalue)
		print("Alternative {} < {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	if not flag:
		print("No significant difference with post-hoc analysis")

def compare_time(valid_users, userinfo_dict, user_task_time):
	num_tasks = 14.0
	performance_groups = {}
	performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	for user in valid_users:
		user_group = userinfo_dict[user]
		performance_groups[user_group].append(user_task_time[user])
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))

	def get_performance_dict(performance_list):
		keys = ['time']
		performance_dict = {}
		for key_ in keys:
			performance_dict[key_] = []
		for tp_performance in performance_list:
			for key_ in keys:
				performance_dict[key_].append(tp_performance)
		return performance_dict

	def compare_all_groups(performance_groups, compared_groups=["control", "concept"]):
		performance_dict = {}
		for group in compared_groups:
			performance_dict[group] = get_performance_dict(performance_groups[group])
		keys = ['time']
		for var_name in keys:
			for group in compared_groups:
				print(len(performance_dict[group][var_name]))
			kwargs = [performance_dict[group][var_name] for group in compared_groups]
			statistic, pvalue = kruskal(*kwargs)
			print(var_name)
			print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
			for group in compared_groups:
				data_list_1 = performance_dict[group][var_name]
				print("Mean: M({}):{:.2f}, SD({}):{:.2f}".format(group, np.mean(data_list_1), group, np.std(data_list_1)))
			if pvalue < 0.05 / 4:
				length = len(compared_groups)
				for i in range(length - 1):
					for j in range(i+1, length):
						group_1 = compared_groups[i]
						group_2 = compared_groups[j]
						data_list_1 = performance_dict[group_1][var_name]
						data_list_2 = performance_dict[group_2][var_name]
						post_hoc_comparison(data_list_1, data_list_2, group_1, group_2)
			print("-" * 17)
	compare_all_groups(performance_groups, compared_groups=["control", "concept", "archie_target", "archie", "archie_od"])
	# return performance_groups


if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")
	user_ATI_scale, user_ATI_time = calc_ATI_scale()

	user_final_task_time = get_user_final_task_time(valid_users, userinfo_dict, user_task_order, user_time_dict)
	user_task_time = compute_task_time(valid_users, user_ATI_time, user_final_task_time)
	compare_time(valid_users, userinfo_dict, user_task_time)

