from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants
from util import compute_time_per_task, get_user_final_task_time, compute_task_time
import pandas as pd
import numpy as np
from scipy.stats import kruskal, mannwhitneyu
from calc_measures import calc_ATI_scale

def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict


def post_hoc_comparison(data_list_1, data_list_2, name1, name2):
	print("Use pots-hoc analysis")
	threshold = 0.05 / 4
	flag = False
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='greater')
	if pvalue < threshold:
		print("Alternative {} > {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	statistic, pvalue = mannwhitneyu(data_list_1, data_list_2, alternative='less')
	if pvalue < threshold:
		print("Alternative {} < {},".format(name1, name2), "pvalue %.4f"%pvalue, "statistic %.4f"%statistic)
		flag = True
	if not flag:
		print("No significant difference with post-hoc analysis")

def compare_time(valid_users, userinfo_dict, user_task_time):
	num_tasks = 14.0
	performance_groups = {}
	performance_groups["control"] = []
	performance_groups["concept"] = []
	performance_groups["archie_target"] = []
	performance_groups["archie"] = []
	performance_groups["archie_od"] = []
	for user in valid_users:
		user_group = userinfo_dict[user]
		performance_groups[user_group].append(user_task_time[user])
	for key_ in performance_groups:
		print(key_, len(performance_groups[key_]))

	def get_performance_dict(performance_list):
		keys = ['time']
		performance_dict = {}
		for key_ in keys:
			performance_dict[key_] = []
		for tp_performance in performance_list:
			for key_ in keys:
				performance_dict[key_].append(tp_performance)
		return performance_dict

	def compare_all_groups(performance_groups, compared_groups=["control", "concept"]):
		performance_dict = {}
		for group in compared_groups:
			performance_dict[group] = get_performance_dict(performance_groups[group])
		keys = ['time']
		for var_name in keys:
			for group in compared_groups:
				print(len(performance_dict[group][var_name]))
			kwargs = [performance_dict[group][var_name] for group in compared_groups]
			statistic, pvalue = kruskal(*kwargs)
			print(var_name)
			print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
			for group in compared_groups:
				data_list_1 = performance_dict[group][var_name]
				print("Mean: M({}):{:.2f}, SD({}):{:.2f}".format(group, np.mean(data_list_1), group, np.std(data_list_1)))
			if pvalue < 0.05 / 4:
				length = len(compared_groups)
				for i in range(length - 1):
					for j in range(i+1, length):
						group_1 = compared_groups[i]
						group_2 = compared_groups[j]
						data_list_1 = performance_dict[group_1][var_name]
						data_list_2 = performance_dict[group_2][var_name]
						post_hoc_comparison(data_list_1, data_list_2, group_1, group_2)
			print("-" * 17)
	compare_all_groups(performance_groups, compared_groups=["control", "concept", "archie_target", "archie", "archie_od"])
	# return performance_groups

def compare_correct_wrong_decision_time(time_list_correct, time_list_wrong):
	print(f"Correct:{len(time_list_correct)}, wrong:{len(time_list_wrong)}")
	statistic, pvalue = kruskal(time_list_correct, time_list_wrong)
	print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
	if pvalue < 0.05 / 4:
		post_hoc_comparison(time_list_correct, time_list_wrong, "correct", "wrong")

def compare_time_across_conditions(time_dict):
	time_list = []
	compared_groups = ["control", "concept", "archie_target", "archie", "archie_od"]
	for condition in compared_groups:
		time_list.append(time_dict[condition])
	statistic, pvalue = kruskal(*time_list)
	print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
	if pvalue < 0.05 / 4:
		length = len(compared_groups)
		for i in range(length - 1):
			for j in range(i+1, length):
				group_1 = compared_groups[i]
				group_2 = compared_groups[j]
				data_list_1 = time_list[i]
				data_list_2 = time_list[j]
				post_hoc_comparison(data_list_1, data_list_2, group_1, group_2)


def check_time_per_correct_decision(valid_users, userinfo_dict, new_user_task_dict, answer_dict, user_time_per_task):
	time_per_correct_decision = {}
	time_per_wrong_decision = {}
	time_correct_decision_per_person = {}
	time_wrong_decision_per_person = {}
	group_users = {}
	for user in valid_users:
		user_group = userinfo_dict[user]
		if user_group not in group_users:
			group_users[user_group] = set()
		group_users[user_group].add(user)
		if user_group not in time_per_correct_decision:
			time_per_correct_decision[user_group] = []
			time_per_wrong_decision[user_group] = []
		if user not in time_correct_decision_per_person:
			time_correct_decision_per_person[user] = []
			time_wrong_decision_per_person[user] = []
		for task_id in answer_dict:
			user_decision = new_user_task_dict[user][(task_id, "advice")]
			time_spent = user_time_per_task[user][task_id]
			AI_advice, ground_truth = answer_dict[task_id]
			if user_decision == ground_truth:
				time_per_correct_decision[user_group].append(time_spent)
				time_correct_decision_per_person[user].append(time_spent)
			else:
				time_per_wrong_decision[user_group].append(time_spent)
				time_wrong_decision_per_person[user].append(time_spent)
	print("-" * 17)
	group_order = ["control", "concept", "archie_target", "archie", "archie_od"]
	print("Avg time per decision")
	print("Condition\tCorrect\tWrong")
	for user_group in group_order:
		x1 = "$%.2f \\pm %.2f$"%(np.mean(time_per_correct_decision[user_group]), np.std(time_per_correct_decision[user_group]))
		x2 = "$%.2f \\pm %.2f$"%(np.mean(time_per_wrong_decision[user_group]), np.std(time_per_wrong_decision[user_group]))
		print("%s & %s & %s& "%(user_group, x1, x2))
	print("-" * 17)
	print("Avg time per person")
	print("Condition\tCorrect\tWrong")
	avg_time_per_person = {}
	avg_time_per_person_wrong = {}
	for user in time_correct_decision_per_person:
		avg_time_per_person[user] = np.mean(time_correct_decision_per_person[user])
		avg_time_per_person_wrong[user] = np.mean(time_wrong_decision_per_person[user])
	correct_time_dict = {}
	wrong_time_dict = {}
	for user_group in group_order:
		tp_group = group_users[user_group]
		tp_time_list = [avg_time_per_person[user] for user in tp_group]
		tp_time_list_ = [avg_time_per_person_wrong[user] for user in tp_group]
		x1 = "$%.2f \\pm %.2f$"%(np.mean(tp_time_list), np.std(tp_time_list))
		x2 = "$%.2f \\pm %.2f$"%(np.mean(tp_time_list_), np.std(tp_time_list_))
		print("%s & %s & %s& "%(user_group, x1, x2))
		# print("%s & %.2f & %.2f\\\\"%(user_group, np.mean(tp_time_list), np.mean(tp_time_list_)))
		correct_time_dict[user_group] = tp_time_list
		wrong_time_dict[user_group] = tp_time_list_
		# compare_correct_wrong_decision_time(time_list_correct=tp_time_list, time_list_wrong=tp_time_list_)
		# print(user_group, "%.2f"%(np.mean(tp_time_list)), "%.2f"%(np.mean(tp_time_list_)))
	print("-" * 17)
	print("Correct Decision Time")
	compare_time_across_conditions(correct_time_dict)
	print("-" * 17)
	print("Wrong Decision Time")
	compare_time_across_conditions(wrong_time_dict)


if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")
	user_ATI_scale, user_ATI_time = calc_ATI_scale()

	# user_final_task_time = get_user_final_task_time(valid_users, userinfo_dict, user_task_order, user_time_dict)
	# user_task_time = compute_task_time(valid_users, user_ATI_time, user_final_task_time)

	user_time_per_task = compute_time_per_task(valid_users, user_ATI_time, userinfo_dict, user_task_order, user_time_dict)
	new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
	
	check_time_per_correct_decision(valid_users, userinfo_dict, new_user_task_dict, answer_dict, user_time_per_task)
	

