from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, load_expertise, load_nasatlx
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance, calc_TiA_scale, calc_ATI_scale
from scipy.stats import kruskal, mannwhitneyu
from pingouin import ancova, anova
from statsmodels.stats.multicomp import pairwise_tukeyhsd
import csv
from collections import Counter

# Mainly Figure 10, Sec 6.4.1, Helpfulness of Explanations and Analogies

def dump_user_feedback(input_file, column_name, output_file, filter_None=True):
	df = pd.read_csv(input_file, usecols=['user_id', column_name], quoting=csv.QUOTE_MINIMAL)
	# , quoting=csv.QUOTE_NONE
	user_data_list = df.values.tolist()
	print(input_file, len(user_data_list))
	user_expertise = {}
	user_cognitive_load = {}
	f_out = open(output_file, "w")
	for user_id, sentence in user_data_list:
		# if filter_None and (sentence.lower() == "none" or sentence is None):
		# 	continue
		f_out.write("%s\t%s\n"%(user_id, sentence))
	f_out.close()

helpfulness_dict = {
	"unhelpful": -2,
	'somewhat unhelpful': -1,
	'neutral': 0,
	'somewhat helpful': 1,
	'helpful': 2
}

def load_opinion(input_file, column_name, userinfo_dict):
	df = pd.read_csv(input_file, usecols=['user_id', column_name], quoting=csv.QUOTE_MINIMAL)
	# , quoting=csv.QUOTE_NONE
	user_data_list = df.values.tolist()
	print(input_file, len(user_data_list))
	user_opinion_dict = {}
	helpfulness_list = []
	for user_id, opinion in user_data_list:
		if userinfo_dict[user_id] in ['archie', 'archie_od']:
			try:
				assert opinion.lower() != "none"
			except:
				print(user_id)
		try:
			if opinion.lower() == "none":
				continue
		except:
			print(user_id, opinion)
		try:
			user_opinion_dict[user_id] = helpfulness_dict[opinion]
		except:
			print(user_id)
		helpfulness_list.append(helpfulness_dict[opinion])
	return user_opinion_dict, helpfulness_list


def pieplot_helpfulness(counter_obj, total, val_range):
	ct_dict = dict(counter_obj)
	print(total)
	data = []
	for ct in val_range:
		ratio = ct_dict[ct] / total
		data.append(ratio)
		print(ct, "%.3f"%(ratio))


	import matplotlib.pyplot as plt
	import seaborn as sns

	#define data
	labels = ['Unhelpful', 'Somewhat Unhelpful', 'Neutral', 'Somewhat Helpful', 'Helpful']

	#define Seaborn color palette to use
	# colors = sns.color_palette('bright')[0:5]
	colors = sns.color_palette('pastel')[0:5]
	sns.set(font="Times New Roman")

	def color_calc():
		# RGB
		color_tuples = [(253,141,60), (199, 160, 133), (252, 240, 225), (251,180,174), (201, 71, 55)]
		# color_tuples = [(229,195,198), (225,233,183), (249,97,97), (188,210,208), (208,183,131)]
		# color_tuples = [(102,194,165), (252,141,98), (141,160,203), (231,138,195), (166,216,84)]
		# color_tuples = [(251,180,174), (179,205,227), (204,235,197), (222,203,228), (254,217,166)]
		float_tuples = []
		for tuple_ in color_tuples:
			tp_tuple = (tuple_[0] / 255.0, tuple_[1] / 255.0, tuple_[2] / 255.0)
			float_tuples.append(tp_tuple)
		return float_tuples

	#create pie chart
	# plt.pie(data, labels = labels, colors = color_calc(), autopct='%.1f%%', textprops={'fontsize': 14})
	print(data)
	# ratios = ["%.1f"%(cell * 100) for cell in data]
	# print(ratios)
	plt.pie(data, labels = labels, colors = color_calc(), autopct='%.1f%%', textprops={'fontsize': 14})
	# plt.legend(labels= labels, loc="center right")
	plt.show()

if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	# new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
	
	user_opinion_explanation, helpfulness_list = load_opinion("anonymized_data/PostQ_ham_2.csv", "opinion_explanation", userinfo_dict)
	explanation_counter = Counter(helpfulness_list)
	total = 0
	for item in [-2, -1, 0, 1, 2]:
		print(item, explanation_counter[item])
		total += explanation_counter[item]
	print(total)
	pieplot_helpfulness(counter_obj=explanation_counter, total=total, val_range=[-2, -1, 0, 1, 2])
	user_opinion_analogy, helpfulness_list = load_opinion("anonymized_data/PostQ_ham_2.csv", "opinion_analogy", userinfo_dict)
	for user in valid_users:
		if userinfo_dict[user] in ['archie', 'archie_od']:
			try:
				user in user_opinion_analogy
			except:
				print(user)
	analogy_counter = Counter(helpfulness_list)
	total = 0
	for item in [-2, -1, 0, 1, 2]:
		print(item, analogy_counter[item])
		total += analogy_counter[item]
	print(total)
	pieplot_helpfulness(counter_obj=analogy_counter, total=total, val_range=[-2, -1, 0, 1, 2])







