from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance
from scipy.stats import kruskal, mannwhitneyu

# Mainly Table 8, Sec 6.4.1, Performance Per Task

def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict

def get_experience_confidence(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	confidence_map = {
		"control": "control_confidence",
		"concept": "concept_confidence",
		"archie_target": "archiet_confidence",
		"archie": "archie_confidence",
		"archie_od": "archieo_confidence"
	}
	user_experience = {}
	user_confidence = {}
	for user in reserved_users:
		user_confidence[user] = {}
		user_experience[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			user_confidence[user][(task_id, "base")] = user_task_dict[user][(task_id, "base_confidence")]
			user_confidence[user][(task_id, "advice")] = user_task_dict[user][(task_id, confidence_map[user_group])]
			user_experience[user][task_id] = user_task_dict[user][(task_id, "base_experience")]
	return user_confidence, user_experience



confidence_scores = {
	'unconfident': -2.0,
	'somewhat unconfident': -1.0,
	'neutral': 0.0,
	'somewhat confident': 1.0,
	'confident': 2.0
}

experience_map = {
	'No': 0.0,
	'Yes': 1.0
}

# accuracy per task
# confidence per task
# experience per task
def check_task_difficulty(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list, user_confidence, user_experience):
	task_accuracy_dict = {}
	user_confidence_dict = {}
	initial_confidence_dict = {}
	user_experience_dict = {}
	initial_accuracy_dict = {}
	new_list = []
	for tp_task in task_ID_list:
		task_accuracy_dict[tp_task] = []
		initial_accuracy_dict[tp_task] = []
		user_confidence_dict[tp_task] = []
		user_experience_dict[tp_task] = []
		initial_confidence_dict[tp_task] = []
		for user in valid_users:
			initial_choice = new_user_task_dict[user][(tp_task, "base")]
			user_choice = new_user_task_dict[user][(tp_task, "advice")]
			AI_advice, ground_truth = answer_dict[tp_task]
			if initial_choice == ground_truth:
				initial_accuracy_dict[tp_task].append(1.0)
			else:
				initial_accuracy_dict[tp_task].append(0.0)
			if user_choice == ground_truth:
				task_accuracy_dict[tp_task].append(1.0)
			else:
				task_accuracy_dict[tp_task].append(0.0)
			user_confidence_dict[tp_task].append(confidence_scores[user_confidence[user][(tp_task, "advice")]])
			user_experience_dict[tp_task].append(experience_map[user_experience[user][tp_task]])
			initial_confidence_dict[tp_task].append(confidence_scores[user_confidence[user][(tp_task, "base")]])
		# print("{}: {:.2f}".format(tp_task, np.mean(task_accuracy_dict[tp_task])))
		new_list.append((tp_task, np.mean(initial_accuracy_dict[tp_task]), np.mean(task_accuracy_dict[tp_task]), np.mean(initial_confidence_dict[tp_task]), np.mean(user_confidence_dict[tp_task]), np.mean(user_experience_dict[tp_task])))
	print(len(new_list))
	new_list.sort(key=lambda x:x[2], reverse=True)
	for task_id, accuracy_ini, accuracy, confidence_ini, confidence, experience in new_list:
		# AI_correctness = "correct" if answer_dict[task_id][0] == answer_dict[task_id][1] else "wrong"
		AI_correctness = "$\\checkmark$" if answer_dict[task_id][0] == answer_dict[task_id][1] else "$\\times$"
		print("{} & {:.3f} & {:.3f} & {:.2f} & {:.2f} & {:.2f} & {} & {}\\\\".format(task_id.replace("_", "-"), accuracy_ini, accuracy, confidence_ini, confidence, experience, answer_dict[task_id][1], AI_correctness))
		# print("{}: Initial accuracy: {:.3f}, Final accuracy: {:.3f}, Initial confidence: {:.2f}, Final confidence: {:.2f}, Experience: {:.2f}, ground truth: {}, AI advice is {}".format(task_id, accuracy_ini, accuracy, confidence_ini, confidence, experience, answer_dict[task_id][1], AI_correctness))
	return new_list

if __name__ == '__main__':
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
	user_task_dict, user_time_dict = read_usertask()
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# approved_participants = load_valid_prolific_ids()
	# print(f"{len(approved_participants)} participants get approved")
	# valid_users = valid_users & approved_participants
	print(f"In total, {len(valid_users)} available for analysis")

	condition_count = {}
	condition_users = {}
	for user in valid_users:
		# if user not in approved_participants:
		# 	continue
		user_group = userinfo_dict[user]
		if user_group not in condition_count:
			condition_count[user_group] = 0
			condition_users[user_group] = set()
		condition_count[user_group] += 1
		condition_users[user_group].add(user)
	for condition in condition_count:
		print(condition, condition_count[condition])

	# standarize the user choice dict and answer dict
	new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	user_confidence, user_experience = get_experience_confidence(valid_users, user_task_dict, userinfo_dict, task_ID_list)
	answer_dict = {}
	for task_id in task_ID_list:
		answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
	avg_list = check_task_difficulty(valid_users, userinfo_dict, new_user_task_dict, answer_dict, task_ID_list, user_confidence, user_experience)
	# condition_task_accuracy = {}
	# for condition in ["control", "concept", "archie_target", "archie", "archie_od"]:
	# 	print("-" * 17)
	# 	print(condition)
	# 	condition_task_accuracy[condition] = check_task_difficulty(condition_users[condition], userinfo_dict, new_user_task_dict, answer_dict, task_ID_list, user_confidence, user_experience)
	# task_order = [item[0] for item in avg_list]
	# print(task_order)
	# task_str_dict = {}
	# # for task in task_order:
	# for task_id, accuracy_ini, accuracy, confidence_ini, confidence, experience in avg_list:
	# 	task_str_dict[task_id] = "{} & {:.3f} & {:.3f} &".format(task_id.replace("_", "-"), accuracy_ini, accuracy)
	# for condition in ["control", "concept", "archie_target", "archie", "archie_od"]:
	# 	tp_list = condition_task_accuracy[condition]
	# 	for task_id, accuracy_ini, accuracy, confidence_ini, confidence, experience in tp_list:
	# 		# AI_correctness = "$\\checkmark$" if answer_dict[task_id][0] == answer_dict[task_id][1] else "$\\times$"
	# 		# print("{:.3f} & {:.3f} ".format(accuracy_ini, accuracy))
	# 		task_str_dict[task_id] += "{:.3f} & {:.3f} &".format(accuracy_ini, accuracy)
	# for task in task_order:
	# 	AI_correctness = "$\\checkmark$" if answer_dict[task][0] == answer_dict[task][1] else "$\\times$"
	# 	task_str_dict[task] += " {} & {}\\\\".format(answer_dict[task][1], AI_correctness)
	# 	# task_str_dict[task] += " \\\\"
	# for task in task_order:
	# 	print(task_str_dict[task])






