from util import load_generation_tasks, read_userinfo, read_usertask, get_valid_participants, load_expertise
import pandas as pd
import numpy as np
from calc_measures import calc_user_reliance_measures, UserPerformance, calc_TiA_scale, calc_ATI_scale
from scipy.stats import kstest


# Sec 6.4.1, Distribution of covariates, Performance Overview


def transform_user_task_dict(reserved_users, user_task_dict, userinfo_dict, task_ID_list):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	new_user_task_dict = {}
	for user in reserved_users:
		new_user_task_dict[user] = {}
		user_group = userinfo_dict[user]
		for task_id in task_ID_list:
			new_user_task_dict[user][(task_id, "base")] = user_task_dict[user][(task_id, "base")]
			new_user_task_dict[user][(task_id, "advice")] = user_task_dict[user][(task_id, answer_type_dict[user_group])]
	return new_user_task_dict



task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
required_condtions = ["control", "concept", "archie_target", "archie", "archie_od"]
user_task_dict, user_time_dict = read_usertask()
userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
# approved_participants = load_valid_prolific_ids()
# print(f"{len(approved_participants)} participants get approved")
# valid_users = valid_users & approved_participants
print(f"In total, {len(valid_users)} available for analysis")

user_ATI_scale, user_ATI_time = calc_ATI_scale()
user_TiA_scale = calc_TiA_scale()
user_expertise = load_expertise(valid_users)

condition_count = {}
for user in valid_users:
	# if user not in approved_participants:
	# 	continue
	user_group = userinfo_dict[user]
	if user_group not in condition_count:
		condition_count[user_group] = 0
	condition_count[user_group] += 1
for condition in condition_count:
	print(condition, condition_count[condition])

# standarize the user choice dict and answer dict
new_user_task_dict = transform_user_task_dict(valid_users, user_task_dict, userinfo_dict, task_ID_list)
answer_dict = {}
for task_id in task_ID_list:
	answer_dict[task_id] = (data_dict[task_id].prediction, data_dict[task_id].ground_truth)
user_TiA_scale = calc_TiA_scale()
TiA_subscales = ["Reliability/Competence", "Understanding/Predictability", "Familiarity", \
				 "Intention of Developers", "Propensity to Trust", "Trust in Automation"]
all_var_dict = {}
dependent_variables = ["accuracy", "agreement_fraction", "switching_fraction", "appropriate_reliance", "RAIR", "RSR"]
for var_name in dependent_variables:
	all_var_dict[var_name] = []
for subscale in TiA_subscales:
	all_var_dict[subscale] = []
all_var_dict["ATI"] = []
all_var_dict["medical_expertise"] = []
all_var_dict["skin_cancer_expertise"] = []
for user in valid_users:
	if user not in user_TiA_scale:
		print(f"User {user} does not have TiA records")
	tp_performance = UserPerformance(username=user)
	user_group = userinfo_dict[user]

	tp_correct, tp_agreement_fraction, tp_switching_fraction, tp_appropriate_reliance, initial_disagreement, relative_positive_ai_reliance, relative_positive_self_reliance = \
	calc_user_reliance_measures(user, usertask_dict=new_user_task_dict, answer_dict=answer_dict, task_id_list=task_ID_list)
	tp_accuracy = tp_correct / 14.0

	all_var_dict["accuracy"].append(tp_accuracy)
	all_var_dict["agreement_fraction"].append(tp_agreement_fraction)
	all_var_dict["switching_fraction"].append(tp_switching_fraction)
	all_var_dict["appropriate_reliance"].append(tp_appropriate_reliance)
	all_var_dict["RAIR"].append(relative_positive_ai_reliance)
	all_var_dict["RSR"].append(relative_positive_self_reliance)
	# all_var_dict["trust"].append(user_trust_first[user])

	all_var_dict["ATI"].append(user_ATI_scale[user])
	all_var_dict["medical_expertise"].append(user_expertise[user]["medical_expertise"])
	all_var_dict["skin_cancer_expertise"].append(user_expertise[user]["skin_cancer_expertise"])
	for subscale in TiA_subscales:
		all_var_dict[subscale].append(user_TiA_scale[user][subscale])
print(len(all_var_dict["accuracy"]), np.mean(all_var_dict["accuracy"]))

for var_name in all_var_dict:
	print(var_name, len(all_var_dict[var_name]), np.mean(all_var_dict[var_name]), np.std(all_var_dict[var_name]))
	ks_statistic, p_value = kstest(all_var_dict[var_name], 'norm')
	print("ks_statistic: {:.2f}, p value: {:.2f}".format(ks_statistic, p_value))
	if p_value < 0.05:
		print("Variable {} is not a normal distribution".format(var_name))
	else:
		print("Null hypo cannot be rejected, Variable {} follows a normal distribution".format(var_name))
	print("-"*17)
