import json
import sys
import os
import pandas as pd
import numpy as np
from datetime import datetime
import csv
# data_folder = "ham_data"
data_folder = "../anonymized_data"

class HamDataset(object):

    def __init__(self, tp_task):
        self.task_id = tp_task['image_id']
        self.image_path = tp_task['path']
        self.cav = tp_task['cav']
        self.score = tp_task['score']
        self.ground_truth = tp_task['ground_truth']
        self.prediction = tp_task['advice']
        self.correctness = tp_task['correctness']
        self.concept_explanation = tp_task['concept_explanation']
        # dict, key: [positive_claim, negative_claim]. value can be null/None
        self.archie_explanation = tp_task['archie_explanation']
        # dict, key: [positive_claim, negative_claim], value list, can be empty list

    @staticmethod
    def deal_int(value):
        if value is not None:
            out_str = "%d"%(value)
        else:
            out_str = "0"
        return out_str

    def print_generation_task(self):
        task_str = ""
        task_str += "Task %s"%(self.task_id)
        task_str += ", with label %s"%(self.prediction)
        task_str += ", with ground truth %s"%(self.ground_truth)
        return task_str

    def serize(self):
        return {
            "task_id": self.task_id,
            "image_path": self.image_path,
            "cav": self.cav,
            "score": self.score,
            "ground_truth": self.ground_truth,
            "prediction": self.prediction,
            "concept_explanation": self.concept_explanation,
            "archie_explanation": self.archie_explanation
        }

def load_generation_tasks(filename="flaskr/static/data/ham_medical/selected_tasks_new.json"):
    f = open(filename)
    data_object = json.load(f)
    task_list = data_object['task_list']
    model_parameter = data_object['model_parameter']
    f.close()
    data_dict = {}
    task_ID_list = []
    for i in range(len(task_list)):
        tp_data = HamDataset(task_list[i])
        data_dict[tp_data.task_id] = tp_data
        task_ID_list.append(tp_data.task_id)
    # print(data_dict['T003'].print_generation_task())
    return task_ID_list, data_dict

# def check_completeness():
# 	complete_flag = True
# 	for task_id in task_ID_list:
# 		required_keys = ['base', 'base_confidence', 'base_experience']
# 		if user_group == "control":
# 			required_keys.append("control")
# 			required_keys.append("control_confidence")
# 		elif user_group == "concept":
# 			required_keys.append("concept")
# 			required_keys.append("concept_confidence")
# 		elif user_group == "archie_target":
# 			required_keys.append("archiet")
# 			required_keys.append("archiet_confidence")
# 		elif user_group == "archie":
# 			required_keys.append("archie")
# 			required_keys.append("archie_confidence")
# 		elif user_group == "archie_od":
# 			required_keys.append("archieo")
# 			required_keys.append("archieo_confidence")
# 		else:
# 			raise NotImplementedError("Unknown condition {}".format(user_gorup))
# 		for tp_key in required_keys:
# 			if (task_id, tp_key) not in user_task_dict[user]:
# 				complete_flag = False
# 				break

def collect_file_users(filename):
	# df = pd.read_csv(filename, usecols=['user_id'])
	df = pd.read_csv(filename, usecols=['user_id'], quoting=csv.QUOTE_MINIMAL)
	user_list = df.values.tolist()
	user_set = set([item[0] for item in user_list])
	# f = open(filename)
	# f.readline()
	# user_set = set()
	# num_lines = 0
	# for line in f:
	# 	prolific_id = line.strip().split(",")[0]
	# 	user_set.add(prolific_id)
	# 	num_lines += 1
	print(filename, len(user_set))
	return user_set

def load_nasatlx(reserved_users):
	df = pd.read_csv(os.path.join(data_folder, "userbehavior.csv"), usecols=['user_id', 'event_desc', 'user_behavior'])
	user_data_list = df.values.tolist()
	user_expertise = {}
	user_cognitive_load = {}
	required_keys = ['mental_demand', 'physical_demand', 'temporal_demand', 'performance', 'effort', 'frustration']
	for user_id, event_desc, user_behavior in user_data_list:
		if event_desc not in required_keys:
			continue
		if user_id not in user_cognitive_load:
			user_cognitive_load[user_id] = {}
		user_cognitive_load[user_id][event_desc] = int(user_behavior)
	for user in reserved_users:
		tp_list = []
		for tp_key in required_keys:
			assert tp_key in user_cognitive_load[user]
			tp_list.append(user_cognitive_load[user][tp_key])
		user_cognitive_load[user]['avg_cognitive_load'] = np.mean(tp_list)
	return user_cognitive_load

def load_expertise(reserved_users):
	df = pd.read_csv(os.path.join(data_folder, "userbehavior.csv"), usecols=['user_id', 'event_desc', 'user_behavior'])
	user_data_list = df.values.tolist()
	user_expertise = {}
	required_keys = ['medical_expertise', 'skin_cancer_expertise']
	for user_id, event_desc, user_behavior in user_data_list:
		if event_desc not in required_keys:
			continue
		if user_id not in user_expertise:
			user_expertise[user_id] = {}
		user_expertise[user_id][event_desc] = int(user_behavior)
	for user in reserved_users:
		for tp_key in required_keys:
			assert tp_key in user_expertise[user]
	return user_expertise

def load_learning_effect(userinfo_dict, reserved_users=None):
	filename = os.path.join(data_folder, "PostQ_ham.csv")
	usecols=['user_id', 'malignant_features', 'benign_features']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_data_list = df.values.tolist()
	user_malignant_features = {}
	user_benign_features = {}
	for user_id, malignant_features, benign_features in user_data_list:
		user_malignant_features[user_id] = malignant_features.split("|")
		user_benign_features[user_id] = benign_features.split("|")
	for user in reserved_users:
		if userinfo_dict[user] == "control":
			continue
		assert user in user_malignant_features
		assert user in user_benign_features
	return user_malignant_features, user_benign_features

def check_nasatlx():
	df = pd.read_csv(os.path.join(data_folder, "userbehavior.csv"), usecols=['user_id', 'event_desc'])
	user_data_list = df.values.tolist()
	user_key_dict = {}
	required_keys = ['medical_expertise', 'skin_cancer_expertise', 'mental_demand', 'physical_demand', 'temporal_demand', 'performance', 'effort', 'frustration']
	required_keys_set = set(required_keys)
	for user_id, event_desc in user_data_list:
		if user_id not in user_key_dict:
			user_key_dict[user_id] = set()
		user_key_dict[user_id].add(event_desc)
	user_set = set()
	for user in user_key_dict:
		if len(required_keys_set - user_key_dict[user]) == 0:
			# all required keys can be found in user_records
			user_set.add(user)
		# else:
		# 	print("User {} missed {}".format(user, required_keys_set - user_key_dict[user]))
	return user_set

def check_completeness(valid_users, user_task_dict, userinfo_dict):
	filenames = ["ATI.csv", "TiA_PostQ.csv", "PostQ_ham.csv", "PostQ_ham_2.csv", "PostQ_ham_3.csv"]
	users_occ_list = []
	for filename in filenames:
		user_set = collect_file_users(os.path.join(data_folder, filename))
		users_occ_list.append(user_set)
		# print(filename, len(user_set))
	#users who have complete records "userbehavior.csv"
	users_occ_list.append(check_nasatlx())
	# reuqired_dict = {
	# 	"control": [0, 1, 4, 5],
	# 	"concept": [0, 1, 2, 4, 5],
	# 	"archie_target": [0, 1, 2, 4, 5],
	# 	"archie": [0, 1, 2, 3, 4, 5],
	# }
	# If only PostQ_ham_3.csv missed, acceptable
	reuqired_dict = {
		"control": [0, 1, 5],
		"concept": [0, 1, 2, 5],
		"archie_target": [0, 1, 2, 5],
		"archie": [0, 1, 2, 3, 5],
		"archie_od": [0, 1, 2, 3, 5],
	}
	valid_user_set = set()
	for user in valid_users:
		if user not in userinfo_dict:
			# user does not belong to any conditions in experiment
			continue
		# check users finished all required questionnaires
		user_group = userinfo_dict[user]
		required_checks = reuqired_dict[user_group]
		flag = False
		for tp_index in required_checks:
			if user not in users_occ_list[tp_index]:
				print("User {} missed {}".format(user, filenames[tp_index]))
				flag = True
				break
		if flag:
			# print(user)
			continue
		# check users finished all tasks and attention checks
		valid_user_set.add(user)
	return valid_user_set

def attention_check_pass(user_id, user_attention_check_correct, user_task_dict):
	if user_id not in user_attention_check_correct:
		user_attention_check_correct[user_id] = 0
	required_keys = [("ATI_attention", 'attention'), ("TiA_attention", 'attention'), ("attention_ham_1", 'choice'),
					("attention_ham_1", 'experience'), ("attention_ham_1", 'confidence')]
	flag = True
	for tp_key in required_keys:
		if tp_key not in user_task_dict[user_id]:
			flag=False
			break
	if flag:
		if user_task_dict[user_id][("ATI_attention", 'attention')] == "6":
			user_attention_check_correct[user_id] += 1
		if user_task_dict[user_id][("TiA_attention", 'attention')] == "1":
			user_attention_check_correct[user_id] += 1
		choice = user_task_dict[user_id][("attention_ham_1", 'choice')]
		experience = user_task_dict[user_id][("attention_ham_1", 'experience')]
		confidence = user_task_dict[user_id][("attention_ham_1", 'confidence')]
		if choice == "benign" and experience == "Yes" and confidence == "neutral":
			user_attention_check_correct[user_id] += 1
	return user_attention_check_correct[user_id] == 3

def task_complete_pass(user_group, user_id, user_task_dict, task_ID_list):
	required_keys = ['base', 'base_confidence', 'base_experience']
	if user_group == "control":
		required_keys.append("control")
		required_keys.append("control_confidence")
	elif user_group == "concept":
		required_keys.append("concept")
		required_keys.append("concept_confidence")
	elif user_group == "archie_target":
		required_keys.append("archiet")
		required_keys.append("archiet_confidence")
	elif user_group == "archie":
		required_keys.append("archie")
		required_keys.append("archie_confidence")
	elif user_group == "archie_od":
		required_keys.append("archieo")
		required_keys.append("archieo_confidence")
	else:
		raise NotImplementedError("Unknown condition {}".format(user_gorup))
	complete_flag = True
	for task_id in task_ID_list:
		if not complete_flag:
			break
		for tp_key in required_keys:
			if (task_id, tp_key) not in user_task_dict[user_id]:
				complete_flag = False
				break
	return complete_flag

def get_complete_participants(user_task_dict, userinfo_dict, task_ID_list):
	# this function will figure out participants who passed all attention checks and have complete task records
	user_attention_check_correct = {}
	complete_users = set()
	valid_users = set()
	for user_id in userinfo_dict:
		if user_id not in user_task_dict:
			print(user_id)
			continue
		user_group = userinfo_dict[user_id]
		if user_id == '6112352c836239c7d189f38e':
			# test prolific ID
			continue
		attention_pass_flag = attention_check_pass(user_id, user_attention_check_correct, user_task_dict)
		# this function will update user_attention_check_correct
		complete_flag = task_complete_pass(user_group, user_id, user_task_dict, task_ID_list)
		if complete_flag:
			complete_users.add(user_id)
			if attention_pass_flag:
				valid_users.add(user_id)
	return complete_users, valid_users
		

def find_valid_participants(userinfo_dict, task_ID_list):
	filename = os.path.join(data_folder, "usertask.csv")
	usecols = ['user_id', 'task_id', 'answer_type', 'choice']
	df = pd.read_csv(filename, usecols=usecols)
	user_task_list = df.values.tolist()

	user_task_dict = {}
	user_attention_check_correct = {}
	for user_id, task_id, answer_type, choice in user_task_list:
		if user_id not in userinfo_dict:
			continue
		user_group = userinfo_dict[user_id]
		if user_id == '6112352c836239c7d189f38e':
			# test prolific ID
			continue
		if user_group == "pilot":
			# ignore participants in pilot study
			continue
		if user_id not in user_task_dict:
			user_task_dict[user_id] = {}
		if user_id not in user_attention_check_correct:
			user_attention_check_correct[user_id] = 0
		if answer_type == "choice":
			if task_id == "attention_ham_1" and choice == "benign":
				user_attention_check_correct[user_id] += 1
			# else:
			# 	print(f"user {user_id} failed attention check ham_1 for choice")

		if answer_type == "experience":
			if task_id == "attention_ham_1" and choice == "Yes":
				user_attention_check_correct[user_id] += 1
			# else:
			# 	print(f"user {user_id} failed attention check ham_1 for experience")

		if answer_type == "confidence":
			if task_id == "attention_ham_1" and choice == "neutral":
				user_attention_check_correct[user_id] += 1
			# else:
			# 	print(f"user {user_id} failed attention check ham_1 for confidence")

		if answer_type == "attention":
			if task_id == "ATI_attention":
				if choice == "6":
					user_attention_check_correct[user_id] += 1
				# else:
				# 	print(user_id, task_id, answer_type, choice)
				# 	print(type(choice))
				# 	print(f"user {user_id} failed attention check ATI, selected {choice}, supposed to be 6")
			
			if task_id == "TiA_attention":
				if choice == "1":
					user_attention_check_correct[user_id] += 1
				# else:
				# 	print(user_id, task_id, answer_type, choice)
				# 	print(f"user {user_id} failed attention check TiA, selected {choice}, supposed to be 1")
		else:
			user_task_dict[user_id][(task_id, answer_type)] = choice
	complete_users = set()
	for user in user_task_dict:
		user_group = userinfo_dict[user]
		if user_group == "pilot":
			# ignore participants in pilot study
			continue
		complete_flag = True
		for task_id in task_ID_list:
			required_keys = ['base', 'base_confidence', 'base_experience']
			if user_group == "control":
				required_keys.append("control")
				required_keys.append("control_confidence")
			elif user_group == "concept":
				required_keys.append("concept")
				required_keys.append("concept_confidence")
			elif user_group == "archie_target":
				required_keys.append("archiet")
				required_keys.append("archiet_confidence")
			elif user_group == "archie":
				required_keys.append("archie")
				required_keys.append("archie_confidence")
			elif user_group == "archie_od":
				required_keys.append("archieo")
				required_keys.append("archieo_confidence")
			else:
				raise NotImplementedError("Unknown condition {}".format(user_gorup))
			for tp_key in required_keys:
				if (task_id, tp_key) not in user_task_dict[user]:
					complete_flag = False
					break
		if complete_flag:
			complete_users.add(user)
	valid_users = set()
	# block_list = ['gaole_pilot', 'gaole_pilot_3', 'agathe_2', 'Ujwal']
	for user in complete_users:
		if user_attention_check_correct[user] >= 5:
			# if user not in block_list:
			valid_users.add(user)
			# print(user, user_attention_check_correct[user])
	return complete_users, valid_users

def load_answers():
	reserved_features = ['Loan_ID', 'Loan_Status', 'Sample_Category']
	answer_file = "../../loan_data_selection/selected_samples.csv"
	answer_df = pd.read_csv(answer_file, usecols=reserved_features)
	answer_list = answer_df.values.tolist()
	answer_dict = {}
	for Loan_ID, Loan_Status, Sample_Category in answer_list:
		tp_answer = "accept" if Loan_Status == "Y" else "reject"
		answer_dict[Loan_ID] = tp_answer
	return answer_dict

def read_userinfo(reserved_users=None, required_condtions=None):
	filename = os.path.join(data_folder, "userinfo.csv")
	usecols = ['user_id', 'task_order_str', 'user_group', 'created_time']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_status_list = df.values.tolist()
	userinfo_dict = {}
	user_task_order = {}
	for user_id, task_order_str, user_group, created_time in user_status_list:
		if created_time.startswith('2023-05-23'):
			# filter all participants from first round
			continue
		if required_condtions is not None:
			if user_group not in required_condtions:
				continue
		userinfo_dict[user_id] = user_group
		user_task_order[user_id] = task_order_str
	return userinfo_dict, user_task_order


def read_usertask(reserved_users=None):
	filename = os.path.join(data_folder, "usertask.csv")
	usecols = ['user_id', 'task_id', 'answer_type', 'choice', 'created_time']
	df = pd.read_csv(filename, usecols=usecols)
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	user_task_list = df.values.tolist()

	user_task_dict = {}
	user_time_dict = {}
	for user_id, task_id, answer_type, choice, created_time in user_task_list:
		if user_id not in user_task_dict:
			user_task_dict[user_id] = {}
			user_time_dict[user_id] = {}
		user_task_dict[user_id][(task_id, answer_type)] = choice
		user_time_dict[user_id][(task_id, answer_type)] = created_time
	return user_task_dict, user_time_dict

def get_user_final_task_time(reserved_users, userinfo_dict, user_task_order, user_time_dict):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	user_final_task_time = {}
	for user in reserved_users:
		tp_order = user_task_order[user]
		user_group = userinfo_dict[user]
		last_task = tp_order.split("|")[-1]
		user_final_task_time[user] = user_time_dict[user][(last_task, answer_type_dict[user_group])]
	return user_final_task_time

def compute_time_per_task(reserved_users, user_ATI_time, userinfo_dict, user_task_order, user_time_dict):
	answer_type_dict = {
		"control": "control",
		"concept": "concept",
		"archie_target": "archiet",
		"archie": "archie",
		"archie_od": "archieo"
	}
	user_time_per_task = {}
	for user in reserved_users:
		user_time_per_task[user] = {}
		tp_order = user_task_order[user]
		user_group = userinfo_dict[user]
		answer_type = answer_type_dict[user_group]
		task_list = tp_order.split("|")
		assert len(task_list) == 14
		for index, tp_task in enumerate(task_list):
			time_end = user_time_dict[user][(tp_task, answer_type)]
			if index == 0:
				time_begin = user_ATI_time[user]
			else:
				last_task = task_list[index-1]
				time_begin = user_time_dict[user][(last_task, answer_type)]
			datetime.fromisoformat(time_end)
			time_end = datetime.fromisoformat(time_end)
			time_begin = datetime.fromisoformat(time_begin)
			time_spent = (time_end-time_begin).total_seconds()
			user_time_per_task[user][tp_task] = time_spent
	return user_time_per_task

def compute_task_time(reserved_users, user_ATI_time, user_final_task_time):
	user_task_time = {}
	for user in reserved_users:
		time_end = user_final_task_time[user]
		time_begin = user_ATI_time[user]
		# time_end = datetime.strptime(time_end, "YYYY-MM-DD H:MI:SS")
		# print(type(time_end))
		time_end = datetime.fromisoformat(time_end)
		time_begin = datetime.fromisoformat(time_begin)
		# time_begin = datetime.strptime(time_begin, "YYYY-MM-DD H:MI:SS")
		time_spent = (time_end-time_begin).total_seconds()
		user_task_time[user] = time_spent
	return user_task_time

def print_user_distribution(user_set, userinfo_dict):
	condition_count = {}
	for user in user_set:
		# if user not in approved_participants:
		# 	continue
		user_group = userinfo_dict[user]
		if user_group not in condition_count:
			condition_count[user_group] = 0
		condition_count[user_group] += 1
	for condition in condition_count:
		print(condition, condition_count[condition])

def load_approved_ids(filename):
	df = pd.read_csv(filename, usecols=['Participant id', 'Status'])
	user_data_list = df.values.tolist()
	user_set = set()
	for user_id, status in user_data_list:
		if status == "APPROVED" or status == "AWAITING REVIEW":
			user_set.add(user_id)
	return user_set

def load_valid_prolific_ids():
	# filenames = ["demographic_data/concept_participants.csv", "demographic_data/control_participants.csv", 
	# "demographic_data/archie_target_participants.csv", "demographic_data/archie_participants.csv",
	# "demographic_data/archie_od_participants.csv"]
	filenames = ["../anonymized_data/prolific_demographics.csv"]
	approved_participants = set()
	for filename in filenames:
		approved_participants |= load_approved_ids(filename)
	return approved_participants

def get_valid_participants(user_task_dict, userinfo_dict, task_ID_list):
	approved_participants = load_valid_prolific_ids()
	print(f"{len(approved_participants)} participants get approved")

	complete_users, valid_users_1 = get_complete_participants(user_task_dict, userinfo_dict, task_ID_list)
	print_user_distribution(complete_users, userinfo_dict)
	# complete_users, participants who finished all tasks
	# valid_users, participants who passed all attention checks
	print(f"{len(complete_users)} participants complete all tasks, among them {len(valid_users_1)} participants passed all attention checks.")
	
	valid_users_1 = valid_users_1 & approved_participants
	print("valid_users_1", len(valid_users_1))
	# print(valid_users_1)
	valid_users_2 = check_completeness(valid_users_1, user_task_dict, userinfo_dict)

	condition_count = {}
	# valid_users = valid_users & approved_participants
	print("valid_users_2", f"In total, {len(valid_users_2)} available for analysis")
	for user in valid_users_2:
		# if user not in approved_participants:
		# 	continue
		user_group = userinfo_dict[user]
		if user_group not in condition_count:
			condition_count[user_group] = 0
		condition_count[user_group] += 1
	for condition in condition_count:
		print(condition, condition_count[condition])

	# valid_users, participants who are qualified for analysis
	# print(f"{len(valid_users_2)} participants are valid for analysis.")
	# print("-"*17)
	# print("participants valid for analysis:")
	# for user in (valid_users_2):
	# 	# print(f"{user} failed one attention check")
	# 	# participants who missed one record in questionnaire or failed attention checks
	# 	if userinfo_dict[user] == "archie_od":
	# 		print(user)
	# print("-"*17)
	# if len(valid_users_2) > len(valid_users_1):
	# 	print("please check following participants:")
	# 	for participant_id in (valid_users_2 - valid_users_1):
	# 		print(participant_id)
	# if len(complete_users) > 0:
	# 	print("-"*17)
	# 	print("participants who also can be approved:")
	# 	for user in complete_users:
	# 		# print(f"{user} failed one attention check")
	# 		# participants who missed one record in questionnaire or failed attention checks
	# 		if userinfo_dict[user] == "archie_target":
	# 			print(user)
	# 	print("-"*17)
	return valid_users_2


if __name__ == "__main__":
	task_ID_list, data_dict = load_generation_tasks("selected_tasks_final.json")
	# required_condtions = ["control", "concept"]
	required_condtions = ["concept"]
	userinfo_dict, user_task_order = read_userinfo(required_condtions=required_condtions)
	user_task_dict, user_time_dict = read_usertask()
	valid_users = get_valid_participants(user_task_dict, userinfo_dict, task_ID_list)
	# complete_users, participants who finished all tasks
	# valid_users, participants who passed all attention checks
	from calc_measures import calc_ATI_scale, calc_TiA_scale
	user_ATI_scale, user_ATI_time = calc_ATI_scale()
	# user_TiA_scale = calc_TiA_scale()
	# reserved_users = set(['5d9c35e740e5c70017ee9a77', '60251455ed25c227a4267c21', 'gaole_pilot'])
	# complete_users, reserved_users = find_valid_participants(userinfo_dict, task_ID_list)
	# print("In total, {} users have complete records, {} users are valid for analysis".format(len(complete_users), len(reserved_users)))
	# for user in reserved_users:
	# 	print(user)
	# print("-"*17)
	# for user in (complete_users - reserved_users):
	# 	# print(f"{user} failed one attention check")
	# 	print(user)
	# print("-"*17)
	# sys.exit(-1)
	# reserved_users = complete_users
	# print(reserved_users)
	user_task_dict, user_time_dict = read_usertask(reserved_users)
	valid_user_set = check_completeness(reserved_users, user_task_dict, userinfo_dict)
	print("In total, {} users have complete records".format(len(valid_user_set)))
	for user in valid_user_set:
		print(user)
	# sys.exit(-1)
	user_final_task_time = get_user_final_task_time(reserved_users, userinfo_dict, user_task_order, user_time_dict)
	user_task_time = compute_task_time(reserved_users, user_ATI_time, user_final_task_time)
	print("Average time: {:.2f} seconds".format(np.mean([user_task_time[user] for user in user_task_time])))
	sys.exit(-1)
	user_accuracy = {}
	correrct_dict = {}
	correrct_dict['total'] = {user_id:0 for user_id in reserved_users}
	for user in reserved_users:
		user_accuracy[user] = []
	for task_id in task_ID_list:
		ground_truth = data_dict[task_id].ground_truth
		if ground_truth not in correrct_dict:
			correrct_dict[ground_truth] = {user_id:0 for user_id in reserved_users}
		for user_id in reserved_users:
			if (task_id, 'pilot') not in user_task_dict[user_id]:
				print(user_id, len(user_task_dict[user_id]))
				# continue
			# assert (task_id, 'pilot') in user_task_dict[user_id]
			user_choice = user_task_dict[user_id][(task_id, 'pilot')]
			if user_choice == ground_truth:
				user_accuracy[user_id].append(1.0)
				correrct_dict['total'][user_id] += 1
				correrct_dict[ground_truth][user_id] +=1
			else:
				user_accuracy[user_id].append(0.0)
	# for user_id in reserved_users:
	# 	print(user_id, np.mean(user_accuracy[user_id]))
	print("average accuracy:", np.mean([user_accuracy[user] for user in reserved_users]))
	print("average malignant accuracy:", np.mean([correrct_dict["malignant"][user] / 4.0 for user in reserved_users]))
	print("average benign accuracy:", np.mean([correrct_dict["benign"][user] / 10.0 for user in reserved_users]))
	# print(correrct_dict['total'])




